package com.supervision.filter;

import com.supervision.util.JwtUtils;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;


@Component
@RequiredArgsConstructor
public class JwtAuthenticationFilter extends OncePerRequestFilter {

    private final JwtUtils jwtUtils;
    private final UserDetailsService userDetailsService;

    @Override
    protected void doFilterInternal(HttpServletRequest request, @NonNull HttpServletResponse response,
                                    @NonNull FilterChain filterChain) throws ServletException, IOException {
        String authHeader = request.getHeader("Authorization");
        if (authHeader != null && authHeader.startsWith("Bearer ")) {
            // 提取JWT Token（去掉前缀"Bearer "）
            String token = authHeader.substring(7);
            String username;
            try {
                // 从JWT中解析用户名
                username = jwtUtils.getUsernameFromToken(token);
            } catch (Exception e) {
                // 如果JWT格式不正确或过期，直接放行（后续的过滤器会处理认证失败）
                filterChain.doFilter(request, response);
                return;
            }
            // 如果成功提取到用户名，并且当前没有已认证的用户
            if (username != null && SecurityContextHolder.getContext().getAuthentication() == null) {
                // 根据用户名从数据库加载用户信息
                UserDetails userDetails = userDetailsService.loadUserByUsername(username);
                // 验证Token的有效性（是否未过期）
                if (!jwtUtils.isTokenExpired(token)) {
                    // 将用户信息封装到Authentication对象中，标记为已认证
                    UsernamePasswordAuthenticationToken authToken =
                            new UsernamePasswordAuthenticationToken(userDetails, null, userDetails.getAuthorities());
                    authToken.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
                    // 将Authentication对象放入SecurityContext，表示当前请求已通过认证
                    SecurityContextHolder.getContext().setAuthentication(authToken);
                }
            }
        }
        filterChain.doFilter(request, response);
    }

}
